import math, random, pygame, copy
pygame.init()
from pygame.locals import *


import myObjects
from myConstants import *
import PW14_variables as customVariables



class You(myObjects.GameObject):  
	def __init__(self, images, lane, size, centerPosition):
		myObjects.GameObject.__init__(self, images, size, centerPosition)  
		self.baseAnimationName='fly'
		self.status='FLY_FW'
		self.lane=lane
		self.laneTarget=lane
		self.flyDelta=0
		
		self.flyFWSpeed=0
		self.flyFWSpeedMAX=10
		self.flyUPSpeed=0
		self.flyUPSpeedMAX=10
		
		self.laser=0
		self.lives=10
		self.wound=-1
		
	def hit(self):
		self.setNextAnimation('hit',1,True)
		killed=False
		self.lives+=self.wound
		print 'lives=',self.lives	
		if self.lives<=0:
			killed=True
		return killed

	def death(self):
		self.setNextAnimation('death',1,True)
		print 'you are dead'
	'''	
	def sumColor(self, newColor):
		mixedColor=pygame.Color('Black')
		if self.color==pygame.Color('White') or\
		newColor in (pygame.Color('White'),pygame.Color('Black')) or\
		newColor==self.color:
			mixedColor=newColor
		else:
			if self.color==pygame.Color('Red'):
				if newColor==pygame.Color('Yellow'):
					mixedColor=pygame.Color('Orange')
				elif newColor==pygame.Color('Blue'):
					mixedColor=pygame.Color('Purple')
			
			elif self.color==pygame.Color('Yellow'):
				if newColor==pygame.Color('Red'):
					mixedColor=pygame.Color('Orange')
				elif newColor==pygame.Color('Blue'):
					mixedColor=pygame.Color('Green')
			
			elif self.color==pygame.Color('Blue'):
				if newColor==pygame.Color('Red'):
					mixedColor=pygame.Color('Purple')
				elif newColor==pygame.Color('Yellow'):
					mixedColor=pygame.Color('Green')
			
			else:
				mixedColor==pygame.Color('Black')
		return mixedColor
	'''

	
class ColorSquareSection(myObjects.GameObject):  
	def __init__(self, color, lane, cell, size, centerPosition):
		images={}
		tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
		self.color=pygame.Color(color.r,color.g,color.b,100)
		tmpSurf.fill(self.color)
		images['stop']=[tmpSurf]
		images['size']=size
		self.lane=lane
		self.cell=cell
		#self.font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 50)
		#images['stop']=[self.font.render(str(cell), True, pygame.Color('Black'),color).convert_alpha()]
		
		myObjects.GameObject.__init__(self, images, size, centerPosition)  
		
		
	def getColor(self):
		return self.color
	
	def setColor(self, color, alpha):
		newColor=pygame.Color(color.r,color.g,color.b,alpha)
		self.images['stop'][0].fill(newColor)
		self.color=newColor

	

class ColorParallelogramSection(myObjects.GameObject):  
	def __init__(self, color, size, centerPosition, angle):
		images={}
		tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
		tmpSurf.fill(color)
		#pygame.draw.rect(tmpSurf, color, (10, 10, 90, 90))
		tmpSurf=pygame.transform.rotate(tmpSurf, angle)
		images['stop']=[tmpSurf]
		images['size']=size
		myObjects.GameObject.__init__(self, images, size, centerPosition)  
		self.color=color
		
	def getColor(self):
		return self.color		

		

class Enemy(myObjects.GameObjectReusable):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		self.color=color
		myObjects.GameObject.__init__(self, images, size, centerPosition)
		self.changeColorToAllFrames(pygame.Color(179,0,92,255),self.color)
		#self.changeColorToAllFrames(pygame.Color('Black'),self.color)
		
		self.lane=lane
		self.cell=cell
		self.influence=influence
		self.flyUPSpeed=0
		self.flyUPSpeedMAX=10
		
		self.health=100
		self.wound=-100		
		self.shot=False
		self.status='NONE'
		
	def getColor(self):
		return self.color  
		
	def hit(self):
		killed=False
		self.health+=self.wound
		if self.health<=0:
			killed=True
		return killed

	def death(self):
		print 'enemy is dead'

	def specialAction(self, argument):
		pass
	
	
class Enemy_Block(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		print 'doing nothing',self.lane,self.cell
		return (None,None)
	
	
class Enemy_Move(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		oldlane=self.lane
		lanesTotalNumber=argument[0]
		options=[-1,1]
		firstTry=random.choice(options)
		#print self.lane,firstTry,self.lane+firstTry
		if self.lane+firstTry<0 or self.lane+firstTry>=lanesTotalNumber:
			options.remove(firstTry)
			self.lane+=options[0]
		self.status='CHANGE_LANE'
		print 'moving',self.lane,self.cell
		return (oldlane,None)
		
		
class Enemy_Shot(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		self.shot=True
		print 'shooting',self.lane,self.cell
		return (None,None)
		
		
class Enemy_ChangeColor(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		color=argument[1]
		self.changeColorToAllFrames(self.color,color)		
		self.color=color
		print 'changing color of enemy',self.lane,self.cell,'to',self.color
		return (None,color)
		
class Enemy_MoveShot(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		#move
		oldlane=self.lane
		lanesTotalNumber=argument[0]
		options=[-1,1]
		firstTry=random.choice(options)
		#print self.lane,firstTry,self.lane+firstTry
		if self.lane+firstTry<0 or self.lane+firstTry>=lanesTotalNumber:
			options.remove(firstTry)
			self.lane+=options[0]
		self.status='CHANGE_LANE'
		print 'moving',self.lane,self.cell
		
		#shot
		self.shot=True
		print 'shooting',self.lane,self.cell		
		
		return (oldlane,None)

class Enemy_MoveChangeColor(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		#move
		oldlane=self.lane
		lanesTotalNumber=argument[0]
		options=[-1,1]
		firstTry=random.choice(options)
		#print self.lane,firstTry,self.lane+firstTry
		if self.lane+firstTry<0 or self.lane+firstTry>=lanesTotalNumber:
			options.remove(firstTry)
			self.lane+=options[0]
		self.status='CHANGE_LANE'
		print 'moving',self.lane,self.cell
		
		#change color
		color=argument[1]
		self.changeColorToAllFrames(self.color,color)		
		self.color=color
		print 'changing color of enemy',self.lane,self.cell,'to',self.color
		
		return (oldlane,color)			
		
class Enemy_ShotChangeColor(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		#shot
		self.shot=True
		print 'shooting',self.lane,self.cell

		#change color
		color=argument[1]
		self.changeColorToAllFrames(self.color,color)		
		self.color=color
		print 'changing color of enemy',self.lane,self.cell,'to',self.color		
		
		return (None,color)
		
class Enemy_Boss(Enemy):
	def __init__(self, images, color, lane, cell, size, centerPosition,influence):
		Enemy.__init__(self, images, color, lane, cell, size, centerPosition,influence)

	def specialAction(self, argument):
		#move
		oldlane=self.lane
		lanesTotalNumber=argument[0]
		options=[-1,1]
		firstTry=random.choice(options)
		#print self.lane,firstTry,self.lane+firstTry
		if self.lane+firstTry<0 or self.lane+firstTry>=lanesTotalNumber:
			options.remove(firstTry)
			self.lane+=options[0]
		self.status='CHANGE_LANE'
		print 'moving',self.lane,self.cell
		
		#shot
		self.shot=True
		print 'shooting',self.lane,self.cell

		#change color
		color=argument[1]
		self.changeColorToAllFrames(self.color,color)		
		self.color=color
		print 'changing color of enemy',self.lane,self.cell,'to',self.color		
		
		return (oldlane,color)		
	
		
class Light(myObjects.GameObject):
	def __init__(self, color, endPosition):
		images={}
		size=endPosition
		tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
		pygame.draw.line(tmpSurf,color,(0,0),endPosition,3)
		images['stop']=[tmpSurf]
		images['size']=size
		myObjects.GameObject.__init__(self, images, size, (endPosition[0]/2,endPosition[1]/2))  
		self.color=color
		
	def getColor(self):
		return self.color  
		
class Laser(myObjects.GameObject):  
	def __init__(self, color, size, centerPosition):
		images={}
		tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
		tmpSurf.fill(color)
		images['stop']=[tmpSurf]
		images['size']=size
		myObjects.GameObject.__init__(self, images, size, centerPosition)  
		self.color=color
		
	def getColor(self):
		return self.color

class Bullet(myObjects.GameObject):  
	def __init__(self, color, size, lane, centerPosition, owner):
		images={}
		tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
		tmpSurf.fill(color)
		images['stop']=[tmpSurf]
		images['size']=size
		myObjects.GameObject.__init__(self, images, size, centerPosition)  
		self.color=color
		self.lane=lane
		self.owner=owner
		
	def getColor(self):
		return self.color
		

class CloseBackGround(myObjects.GameObject):
	def __init__(self, images, size, topleft):
		centerPosition=topleft[0]+size[0]/2,topleft[1]+size[1]/2
		myObjects.GameObject.__init__(self, images, size, centerPosition)
		
class FarBackGround(myObjects.GameObject):
	def __init__(self, images, size, topleft):
		centerPosition=topleft[0]+size[0]/2,topleft[1]+size[1]/2
		myObjects.GameObject.__init__(self, images, size, centerPosition)

class Overprint(myObjects.GameObject):
	def __init__(self, screenSize, size, centerPosition):
		images={}
		tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
		color=pygame.Color('Grey')
		color.a=200
		tmpSurf.fill(color)
		images['stop']=[tmpSurf]
		images['size']=size
		myObjects.GameObject.__init__(self, images, size, centerPosition)	
		
class PauseScreen(Overprint):
	def __init__(self, screenSize, colorsImage, level, MadColorImage=None):
		self.level=level
		self.MadColorImage=MadColorImage
		self.size=screenSize[0]*8/10,screenSize[1]*8/10
		if self.level == LEVEL2:
			self.colorsImage=colorsImage['2'][0]
		elif self.level in (LEVEL3, MADCOLORS):
			self.colorsImage=colorsImage['3'][0]
		centerPosition=screenSize[0]/2,screenSize[1]/2
		Overprint.__init__(self, screenSize, self.size, centerPosition)
		self.update()
	
	def update(self):
		font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 15)
		texts=[]
		texts.append('CONTROLS:')
		texts.append('move_up: '+str(pygame.key.name(customVariables.move_up)))
		texts.append('move_down: '+str(pygame.key.name(customVariables.move_down)))
		for t in customVariables.shot_triple:
			key=[]
			for k in t:
				key.append(str(pygame.key.name(k)))
			texts.append('shot_triple: '+str(key))
		texts.append('(any of the combination above works)')
		texts.append('Customize controls by editing PW14_variables.py')
		texts.append('')
		texts.append('Kill all the enemies by shooting them with bullets of different color.')
		texts.append('Bullets can pass through cells "not containing" the bullet color.')
		if self.level>LEVEL0:
			texts.append('Refer to the color map on the right for color addition logic.')
		texts.append('Press any key to continue. PRESS ESC TO QUIT.')
		for t in range(len(texts)):
			line=font.render(texts[t], True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
			self.image.blit(line, (self.size[0]/25,(t+1)*self.size[1]/15))
		if self.level==LEVEL2:
			self.image.blit(self.colorsImage, (self.size[0]*13/20,self.size[0]*1/20))
		elif self.level==LEVEL3:
			self.image.blit(self.colorsImage, (self.size[0]*13/20,self.size[0]*1/20))
		elif self.level==MADCOLORS:
			self.image.blit(self.MadColorImage, (self.size[0]*13/20,self.size[0]*1/20))
		
			
class CountDown(PauseScreen):
	def __init__(self, screenSize, colorsImage, level, startCounter):
		self.counter=startCounter*FPS
		self.countdown=True
		self.start=False
		PauseScreen.__init__(self, screenSize, colorsImage, level)

	def update(self):
		if self.counter>FPS:
			font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 50)
			num=font.render(str(self.counter/FPS), True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
			self.image.blit(num, (self.size[0]/3,self.size[1]/3))
			self.counter-=1
		elif self.counter>0:
			font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 50)
			num=font.render(str('GO!'), True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
			self.image.blit(num, (self.size[0]/3,self.size[1]/3))
			self.counter-=1
		else:
			self.start=True

			
class MadColorCountDown(PauseScreen):
	def __init__(self, screenSize, colorsImage, level, startCounter):
		self.colorsImage=colorsImage
		self.orderedColors=[pygame.Color('Red'),pygame.Color('Purple'),pygame.Color('Blue'),pygame.Color('Cyan'),pygame.Color('Green'),pygame.Color('Yellow'),pygame.Color('White'),pygame.Color('Black')]		
		self.randomColors=[pygame.Color('Red'),pygame.Color('Purple'),pygame.Color('Blue'),pygame.Color('Cyan'),pygame.Color('Green'),pygame.Color('Yellow'),pygame.Color('White'),pygame.Color('Black')]		
		#random.shuffle(self.randomColors)
		self.counter=startCounter*FPS
		self.countdown=False
		self.changeColors=True
		self.start=False
		PauseScreen.__init__(self, screenSize, colorsImage, level)
	
	def update(self):
		if not self.countdown:
			font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 15)
			texts=[]
			texts.append('Bonus Level')
			texts.append('ARTICA - SIX MONTH LATER')
			texts.append('')
			texts.append('Colors totally gone mad!')
			texts.append('Press ENTER to randomize again the colors')
			texts.append('')
			texts.append('Press any key to play with these colors')
			texts.append('Press ESC to quit')
			for t in range(len(texts)):
				line=font.render(texts[t], True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
				self.image.blit(line, (self.size[0]/25,(t+1)*self.size[1]/15))

			if self.changeColors:
				self.changedImage = pygame.PixelArray(copy.copy(self.colorsImage))
				random.shuffle(self.randomColors)
				tolerance=0
				for c in range(len(self.orderedColors)):
					toColor=self.randomColors[c]
					toColor.a=254
					#print 'change color from',self.orderedColors[c],'to',toColor
					self.changedImage.replace(self.orderedColors[c], toColor, tolerance)
				for fromCol in self.randomColors:
					toCol=pygame.Color(fromCol.r,fromCol.g,fromCol.b,255)
					self.changedImage.replace(fromCol, toCol)
				self.changeColors=False
			self.image.blit(self.changedImage.make_surface(), (self.size[0]*13/20,self.size[0]*1/20))
		else:
			if self.counter>FPS:
				font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 50)
				num=font.render(str(self.counter/FPS), True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
				self.image.blit(num, (self.size[0]/3,self.size[1]/3))
				self.counter-=1
			elif self.counter>0:
				font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), 50)
				num=font.render(str('GO!'), True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
				self.image.blit(num, (self.size[0]/3,self.size[1]/3))
				self.counter-=1
			else:
				self.start=True


